#ifndef SRC_SUPPORT_H
#define SRC_SUPPORT_H

#include "../config.h"
#include<gtk/gtk.h>
#include<glib/gprintf.h>

typedef struct _StegoState
{
  gchar state;//'E' for encrypt 'D' for decrypt  
  gboolean  src_file;//TRUE if src file selected.
  gboolean  des_file;//TRUE if des file selected.
} StegoState;


GtkWidget* lookup_widget                          (GtkWidget       *widget,
                                        const gchar     *widget_name);

GtkWidget* create_window(void);

gboolean stego_change_state(StegoState *s,GtkWidget *root);
gboolean stego_report_dialog(gchar *str);

/*
  g_free the pointer returned.
*/
gchar  *stego_get_file(const gchar *title);
gchar  *stego_get_file_ensure(const gchar *title,const gchar *type);

#endif /* SRC_SUPPORT_H */
